// (C) SLESH
unit SSLSocket;

interface
uses winsock;

type
TSLESH_SSL_Socket = class
  function Connect(ip:string; port:word):boolean;
  function Send(buf:pchar; len:integer):integer;
  function Recv(buf:pchar; len:integer):integer;
  procedure Close;
  constructor Create(InitWinSock:boolean);
  destructor Destroy; override;

 private
  ws:WSAData;
  method:pointer;
  ctx:pointer;
  ssl:pointer;
  caddr:sockaddr_in;
 public
  sock: Tsocket; // .      
  ErrorFlag:boolean; //  TRUE     
  Connected:boolean; //  TRUE   
end;


implementation
 const SSL_LIB_NAME = 'ssleay32.dll';

function SSL_library_init:Integer; cdecl; external SSL_LIB_NAME;
function SSLv2_client_method:Pointer; cdecl; external SSL_LIB_NAME;
function SSL_CTX_new(meth: Pointer):Pointer; cdecl; external SSL_LIB_NAME;
function SSL_new(ctx: Pointer):Pointer; cdecl; external SSL_LIB_NAME;
function SSL_set_fd(s: pointer; fd: tsocket):Integer cdecl; external SSL_LIB_NAME;
function SSL_connect(ssl: pointer):Integer; cdecl; external SSL_LIB_NAME;
procedure SSL_free(ssl: pointer); cdecl; external SSL_LIB_NAME;
function SSL_read(ssl: pointer; buf: PChar; num: Integer):Integer; cdecl; external SSL_LIB_NAME;
function SSL_write(ssl: pointer; buf: PChar; num: Integer):Integer; cdecl; external SSL_LIB_NAME;

//  . :
//      
function TSLESH_SSL_Socket.Send(buf:pchar; len:integer):integer;
begin
  if Connected then
    result := SSL_write(ssl, buf, len)
  else
    result := 0;
end;

//  . :
//     
function TSLESH_SSL_Socket.Recv(buf:pchar; len:integer):integer;
begin
  if Connected then
    result := SSL_read(ssl, buf, len)
  else
    result := 0;
end;

//  
procedure TSLESH_SSL_Socket.Close;
begin
  if sock <> INVALID_SOCKET then
  begin
    closesocket(sock);
    sock := INVALID_SOCKET;
  end;

  Connected := false;
end;

//   .:
// ip    
//   .  443
function TSLESH_SSL_Socket.Connect(ip:string; port:word):boolean;
var
  nip:integer;
  phe:PHostEnt;
begin
  result := false;
  sock := socket(AF_INET, SOCK_STREAM, 0);
  ErrorFlag := sock = INVALID_SOCKET;
  //   
  if (ErrorFlag = false) then
  begin
    nip := inet_addr(pansichar(ip));
    if nip = INADDR_NONE then //       IP
    begin
      //  
      phe := gethostbyname(pansichar(ip));
      if phe <> nil then
      begin
        nip := integer(pointer(phe^.h_addr^)^);
      end;
    end;

     //     
    if nip <> INADDR_NONE then
    begin
      caddr.sin_family := AF_INET;
      caddr.sin_addr.s_addr := nip;
      caddr.sin_port := htons(port);

      // 
      if winsock.connect(sock, caddr, sizeof(caddr)) <> SOCKET_ERROR then
      begin
        //     SSL
        SSL_set_fd(ssl, sock);
        SSL_connect(ssl);
        result := true;
      end;
    end;
  end;

  Connected := result;
end;


//  
//   InitWinSock = true    winsock
constructor TSLESH_SSL_Socket.Create(InitWinSock:boolean);
begin
  ErrorFlag := true;
  Connected := false;
  if InitWinSock then
  begin
    //  winsock
    WSAStartUp($101, ws);
  end;

  //  SSL 
  SSL_library_init();
  method := SSLv2_client_method();
  if method <> nil then
  begin
    //     SSL
    ctx := SSL_CTX_new(method);
    if ctx <> nil then
    begin
      ssl := SSL_new(ctx);
      if ssl <> nil then
      begin
        ErrorFlag := false;
      end;
    end;
  end;
end;

// 
destructor TSLESH_SSL_Socket.Destroy;
begin
  Close; //  
  if ssl <> nil then SSL_free(ssl); //  
end;

end.
